<?php

/** @generate-class-entries */

/**
 * @var int
 * @cvalue E_ERROR
 */
const E_ERROR = UNKNOWN;

/**
 * @var int
 * @cvalue E_WARNING
 */
const E_WARNING = UNKNOWN;

/**
 * @var int
 * @cvalue E_PARSE
 */
const E_PARSE = UNKNOWN;

/**
 * @var int
 * @cvalue E_NOTICE
 */
const E_NOTICE = UNKNOWN;

/**
 * @var int
 * @cvalue E_CORE_ERROR
 */
const E_CORE_ERROR = UNKNOWN;

/**
 * @var int
 * @cvalue E_CORE_WARNING
 */
const E_CORE_WARNING = UNKNOWN;

/**
 * @var int
 * @cvalue E_COMPILE_ERROR
 */
const E_COMPILE_ERROR = UNKNOWN;

/**
 * @var int
 * @cvalue E_COMPILE_WARNING
 */
const E_COMPILE_WARNING = UNKNOWN;

/**
 * @var int
 * @cvalue E_USER_ERROR
 */
const E_USER_ERROR = UNKNOWN;

/**
 * @var int
 * @cvalue E_USER_WARNING
 */
const E_USER_WARNING = UNKNOWN;

/**
 * @var int
 * @cvalue E_USER_NOTICE
 */
const E_USER_NOTICE = UNKNOWN;

/**
 * @var int
 * @cvalue E_STRICT
 */
const E_STRICT = UNKNOWN;

/**
 * @var int
 * @cvalue E_RECOVERABLE_ERROR
 */
const E_RECOVERABLE_ERROR = UNKNOWN;

/**
 * @var int
 * @cvalue E_DEPRECATED
 */
const E_DEPRECATED = UNKNOWN;

/**
 * @var int
 * @cvalue E_USER_DEPRECATED
 */
const E_USER_DEPRECATED = UNKNOWN;

/**
 * @var int
 * @cvalue E_ALL
 */
const E_ALL = UNKNOWN;
